const UserFlow = {
	/**
	 * Checks if the logged-in user has access to a specified role for a given store.
	 * @param {Object} options - An object containing role and store information.
	 * @param {string} options.role - The role to check access for.
	 * @param {string} options.storeString - A string containing store IDs.
	 * @returns {boolean} - True if the logged-in user has access to the role, otherwise false.
	 */
	hasAccessToRole: async function ({ role, storeString }) {
		if (!storeString) return false;

		const stores =
			storeString.indexOf(',') > -1
				? storeString.replace(/\s/g, '').toLowerCase().split(',')
				: [storeString.trim().toLowerCase()];
		const roleLevel = Utils.getUserRoleByName(role)?.level;

		const { brand, countryCode } = await State.getState(StateKeys.CURRENT_USER);

		let hasAccess = false;

		try {
			for (const sId of stores) {
				const { storesAndRoles } = await UserFlow.getUserRoleByStore({
					brand,
					countryCode,
					storeId: sId,
				});

				if (storesAndRoles?.length) {
					for (const ur of storesAndRoles) {
						const loggedInUserRole = Utils.getUserRoleByName(ur?.role?.name);
						if (loggedInUserRole?.level > roleLevel) {
							hasAccess = true;
							break;
						}
					}
				}

				if (hasAccess) break;
			}
		} catch (error) {
			console.error('Error checking access to role:', error);
		}

		return hasAccess;
	},
	/**
	 * Groups user stores by role asynchronously.
	 * @param {Object} user - The user object containing user roles.
	 * @param {string[]} user.userRoles - An array of user roles.
	 * @returns {Object} - An object representing grouped user stores by role.
	 */
	groupUserStoresByRole: async function (userRoles) {
		if (!userRoles) return {};
		let storesAndRoles = [];
		const { data: rolesMap } = Cache.getCache(CacheKeys.ROLE_DEFINITIONS);

		const transformedRoles = await Promise.all(
			userRoles.map(async roleString => {
				const [_brand, _country, , _store, , _app, , _role] =
					roleString.split('-');

				const brand = Utils.brandsMap[_brand];
				const { data: countrySpecifics } = Cache.getCache(
					CacheKeys.COUNTRY_DATA
				);
				const country = countrySpecifics[_country];

				const { address = '', city = '' } = await UserFlow.getSpecificStoreData(
					{
						brand: _brand,
						countryCode: _country,
						storeId: _store,
					}
				);

				const store = {
					id: _store.toLowerCase(),
					name: _store.toUpperCase(),
					address,
					city,
				};
				const { name: role } = rolesMap[_role];

				const roleData = {
					brand,
					country,
					store,
					role,
				};

				return roleData;
			})
		);

		const filteredRoles = transformedRoles.reduce((acc, curr) => {
			const existingRole = acc.find(role => role.store.id === curr.store.id);
			const currRoleLevel = Utils.getUserRoleByName(curr.role).level;
			const exisitingRoleLevel = existingRole
				? Utils.getUserRoleByName(existingRole.role).level
				: 0;
			if (currRoleLevel > exisitingRoleLevel) {
				const index = acc.indexOf(existingRole);
				if (index !== -1) {
					acc.splice(index, 1, curr);
				} else {
					acc.push(curr);
				}
			}
			return acc;
		}, []);

		storesAndRoles = await UserFlow.createStoresAndRolesArray(filteredRoles);

		return {
			roles: await UserFlow.groupStoresByRole({ roles: filteredRoles }),
			storesAndRoles,
		};
	},
	/**
	 * Retrieves the email address from a user object, prioritizing the 'eMail' property
	 * and falling back to 'userPrincipalName' if 'eMail' is not available.
	 * @param {Object} user - The user object from which to retrieve the email address.
	 * @returns {string} The email address retrieved from the user object, or an empty string if not found.
	 */
	getEmailFromUser: function (user) {
		const { eMail = '', userPrincipalName = '' } = user;
		const formattedMail = (
			eMail ? eMail : Utils.isEmail(userPrincipalName) ? userPrincipalName : ''
		)?.toLowerCase();
		return formattedMail;
	},
	/**
	 * Updates the list of stores in the edit user drawer based on the user's role.
	 * @param {Object} options - An object containing drawer and list configuration.
	 * @param {HTMLElement} options.drawerClass - The class of the drawer element.
	 * @param {string} options.listItemType - The type of list item to display.
	 * @param {boolean} options.disableCurrentStores - Whether to disable current stores in the list.
	 * @returns {Promise<boolean>} - A promise that resolves to true if the update is successful, otherwise false.
	 */
	updateEditUserDrawerStoresList: async function ({
		drawerClass,
		listItemType,
		disableCurrentStores = false,
	}) {
		const { storesAndRoles, role, storesToAdd, storesToRemove } =
			await State.getSingleOrMultipleToAddUser();

		const { isStoreLensAdmin } = await State.getState(StateKeys.CURRENT_USER);

		if (!role) return;

		const uniqueStores = new Set();

		const stores = await Promise.all(
			[...(storesAndRoles ?? []), ...(storesToAdd ?? [])]
				.filter(sr => sr?.role?.name === role?.name)
				.filter(sr => {
					const storeId = sr?.store?.id;
					if (uniqueStores.has(storeId)) {
						return false;
					}
					uniqueStores.add(storeId);
					return true;
				})
				.map(async s => {
					const existingStore = StoreHelper.storeExistInArray({
						storeToCheck: s?.store,
						stores: storesAndRoles,
					});
					return await StoreHelper.getStoreWithAccessControl({
						...s,
						role,
						existingStore,
						isStoreLensAdmin,
					});
				})
		);

		const drawerElement = document.querySelector(drawerClass);
		const submitButton = drawerElement.querySelector('.submit-button');

		Utils.toggleSubmitButton({
			button: submitButton,
			disabled: !storesToAdd?.length && !storesToRemove?.length,
		});

		await DomHelper.updateStoresList({
			drawerClass,
			stateUser: StateKeys.USER_TO_MANAGE,
			stores,
			listItemType,
			disableCurrentStores,
		});
	},
	getUserRoleByStore: async function ({ brand, countryCode, storeId }) {
		const body = {
			level: UserAccessInfoLevels.ROLES,
			brand,
			countryCode,
			storeId: storeId.toLocaleLowerCase(),
		};
		const { data: userRoles = [] } = await Cache.cachedFunction(
			Cache.constructUserAccessInfoCacheKey(body),
			1,
			async () => await Api.getUserAccessInfo(body)
		);

		const { roles, storesAndRoles } = await UserFlow.groupUserStoresByRole(
			userRoles
		);

		return { roles, storesAndRoles };
	},
	updateUserProfileDrawerRolesAndStores: async function ({
		drawerElement,
		roles,
	}) {
		const $rolesListWrapper = drawerElement.querySelector(
			'.roles-list-wrapper'
		);
		// const $applicationsWrapper = drawerElement.querySelector(
		// 	'.applications-wrapper'
		// );
		const { multipleAdd } = await State.getState(StateKeys.USERS);
		const $roleList = drawerElement.querySelector('#roles-list');
		let toAddString = '';
		const { data: rolesMap } = Cache.getCache(CacheKeys.ROLE_DEFINITIONS);

		for (const array of Object.values(roles)) {
			let role = '';
			let stores = '';

			if (Array.isArray(array)) {
				for (const r of array) {
					const { store: _store = '', role: _role = '' } = r;
					role = role ? role : _role;
					stores += `${(stores?.length ? ', ' : '').trim()} ${_store?.name}`;
				}
			}

			const roleId = (typeof role === 'object' ? role.name : role)
				?.toLowerCase()
				?.replace(' ', '-');
			const roleObj = rolesMap[roleId];

			if (!roleId || !role) continue;

			const buttonAndMenu = !multipleAdd
				? `<button class="mdc-icon-button material-icons elevate-item">
							<div class="mdc-icon-button__ripple"></div>
							more_horiz
						</button>
						<div class="mdc-menu mdc-menu-surface">
							<ul
								role="menu"
								aria-hidden="true"
								aria-orientation="vertical"
								class="mdc-list"
							>
								<li
									role="menuitem"
									class="flex items-center h-10 mdc-list-item"
								>
									<span class="mdc-list-item__ripple"></span>
									<button
										class="mdc-list-item__text add-user-add-values-trigger w-full text-left"
										data-statetoAdd-usertoadd-role='${JSON.stringify(roleObj)}'
									>
										Edit
									</button>
								</li>
							</ul>
						</div>`
				: '';

			toAddString +=
				SEPARATOR +
				`<li
					z-index="0"
					class="items-center gap-5 py-2 !px-0 mdc-list-item bt-0 mdc-menu-surface--anchor"
				>
					<div class="flex justify-between w-full">
						<span class="text-sm mdc-list-item__text mdc-theme-primary">
							<span
								class="mdc-list-item__secondary-text text-zinc-500 mdc-typography--caption"
								>Role
							</span>
							<span
								class="mdc-list-item__primary-text mdc-typography--subtitle2 font-normal"
							>
								${roleObj.name}
							</span>
						</span>
						${buttonAndMenu}
					</div>
				</li>
				<li
					z-index="0"
					class="items-center gap-5 py-2 !px-0 mdc-list-item bt-0 pt-0 mdc-menu-surface--anchor"
				>
					<div class="flex justify-between w-full">
						<span class="text-sm mdc-list-item__text mdc-theme-primary">
							<span
								class="mdc-list-item__secondary-text text-zinc-500 mdc-typography--caption"
								>Stores
							</span>
							<span
								class="mdc-list-item__primary-text mdc-typography--subtitle2 font-normal"
							>
								${stores}
							</span>
						</span>
					</div>
				</li>` +
				SEPARATOR;
		}

		$roleList.innerHTML = '';

		$roleList.insertAdjacentHTML('beforeend', toAddString);

		Utils.toggleClasses({
			add: !toAddString.length,
			classesToToggle: 'hidden',
			element: $rolesListWrapper,
		});

		// Utils.toggleClasses({
		// 	add: !toAddString.length,
		// 	classesToToggle: 'hidden',
		// 	element: $applicationsWrapper,
		// });

		initMDCMenus();
		await updateDrawers();
	},
	updateUserProfileDrawerButtons: async function () {
		const drawerElement = document.querySelector(
			DrawerClasses.ADD_USER__USER_PROFILE
		);
		const { userToAdd, multipleAdd = false } = await State.getState(
			StateKeys.USERS
		);
		const { storesToAdd = [], storesToRemove = [] } = userToAdd;
		const hasStores = !!storesToAdd?.length || !!storesToRemove?.length;

		const $submitButton = drawerElement.querySelector('.submit-button');

		const $multipleAddSubmitButton = drawerElement.querySelector(
			'.multiple-add-user-button'
		);
		const $singleAddUserSaveButton = drawerElement.querySelector(
			'.single-add-save-button'
		);

		Utils.toggleClasses({
			add: !multipleAdd,
			classesToToggle: 'hidden',
			element: $multipleAddSubmitButton,
		});
		Utils.toggleClasses({
			add: multipleAdd || hasStores,
			classesToToggle: 'hidden',
			element: $submitButton,
		});
		Utils.toggleClasses({
			add: multipleAdd || !hasStores,
			classesToToggle: 'hidden',
			element: $singleAddUserSaveButton,
		});
	},
	setUpButtonWithDialog: async function ({ button, title, text, action }) {
		if (!button) return false;
		const dialog = new Dialog({
			title,
			content: `<p class="mdc-theme--primary mdc-typography--body1">${text}</p>`,
			confirmButtonText: 'OK',
			closeButtonText: 'Cancel',
			confirmAction: () => action(),
		});
		button.addEventListener('click', () => {
			dialog.open();
		});
	},
	/**
	 * Retrieves countries data for a given brand asynchronously.
	 * @param {string} brand - The brand for which to retrieve countries data.
	 * @returns {Promise<Object>} - A promise that resolves to an object containing countries data.
	 */
	getCountriesData: async function (brand) {
		const { data: countries = [] } = await Cache.cachedFunction(
			`allCountriesByBrand-${brand}`,
			0.5,
			async () => await Api.getCountries({ brand })
		);

		const countriesArray = countries?.map(c => {
			const { data: countrySpecifics } = Cache.getCache(CacheKeys.COUNTRY_DATA);
			const country = countrySpecifics[c];
			return { name: country ? country?.name : '', id: c };
		});

		return { countries, countriesArray };
	},
	/**
	 * Retrieves stores data for a given brand and country code asynchronously.
	 * @param {string} brand - The brand for which to retrieve stores data.
	 * @param {string} countryCode - The country code for which to retrieve stores data.
	 * @returns {Promise<Object>} - A promise that resolves to an object containing stores data.
	 */
	getStoresData: async function (brand, countryCode) {
		const { data: stores = [] } = await Cache.cachedFunction(
			Cache.constructStoreSetupCacheKey({
				displayOption: GetStoresDisplayOptions.ALL,
				brand,
				countryCode,
			}),
			0.5,
			async () =>
				await Api.getAllStores({
					brand,
					countryCode,
				})
		);
		const storesArray = await Promise.all(
			stores?.map(async ({ StoreCode = '', Address = {}, City = '' }) => {
				return {
					name: StoreCode?.toUpperCase(),
					address: Address?.StreetName1,
					city: City,
					id: StoreCode,
				};
			})
		);

		return { stores, storesArray };
	},
	getSpecificStoreData: async function ({ brand, countryCode, storeId }) {
		if (!brand || !countryCode) {
			return {
				address: '',
				city: '',
			};
		}

		const { data: stores = [] } = await Cache.cachedFunction(
			Cache.constructStoreSetupCacheKey({
				displayOption: GetStoresDisplayOptions.ALL,
				brand,
				countryCode,
			}),
			0.5,
			async () =>
				await Api.getAllStores({
					brand,
					countryCode,
				})
		);

		const store = stores?.find(
			s => s.StoreCode.toLocaleLowerCase() === storeId.toLocaleLowerCase()
		);

		if (!store) {
			return {
				address: '',
				city: '',
			};
		}
		const { Address = {}, City: city = '' } = store;
		return {
			address: Address?.StreetName1,
			countryCode,
			city,
			storeId,
			brand,
		};
	},
	populateSetAndHandleSelectChange: async function ({ selects, data }) {
		return await Promise.all(
			data.map(
				async ({
					selector,
					id,
					values,
					callback,
					ariaLabel,
					loader,
					neverDisable = false,
				}) => {
					const select = selects.find(s => s.id === selector);

					if (select && values?.length) {
						await DomHelper.updateSelect({
							select,
							values,
							id,
							ariaLabel,
							neverDisable,
							callback,
							selects,
							loader,
						});
					}
				}
			)
		);
	},
	createStoresAndRolesArray: async function (roles) {
		return await Promise.all(
			roles?.map(async r => {
				const { role, store, country } = r;
				const roleKey = Utils.getRoleKeyByName(role);
				const countryKey = Utils.getCountryKeyByName(country.name);
				const hasAccess = false;

				return {
					...r,
					role: {
						name: role,
						id: roleKey,
					},
					country: {
						name: country.name,
						id: countryKey,
					},
					hasAccess,
				};
			})
		);
	},
	groupStoresByRole: async function ({ roles }) {
		if (!roles) return {};
		return roles.reduce((acc, curr) => {
			const key = typeof curr.role === 'string' ? curr.role : curr.role.name;
			acc[key] = [...(acc[key] || []), curr];
			return acc;
		}, {});
	},
	storeCheckBoxListenerFunction: async (
		e,
		drawerElement,
		drawerClass,
		stateUser,
		listItemType
	) => {
		{
			let noUpdate = false;
			const { checked, id } = e.target;

			const storeId =
				Utils.extractStoreIdFromCheckBoxId(id).toLocaleLowerCase();

			const { storesToAdd = [], role } =
				await State.getSingleOrMultipleToAddUser();

			const { brand, country: countryCode } = await Utils.getSelectsValues({
				drawerElement,
			});

			const { address = '', city = '' } = await UserFlow.getSpecificStoreData({
				brand,
				countryCode,
				storeId,
			});

			const { data: countryData } = Cache.getCache(CacheKeys.COUNTRY_DATA);
			const country = countryData[countryCode];

			let updatedStoresToAdd = [];

			if (checked) {
				const storeToAdd = {
					brand: Utils.brandsMap[brand],
					country: {
						id: countryCode,
						name: country.name,
					},
					hasAccess: true,
					role,
					store: { id: storeId, name: storeId.toUpperCase(), address, city },
				};
				updatedStoresToAdd = [...storesToAdd, storeToAdd];
			} else {
				const store = storesToAdd.find(
					s => s.store.id.toLocaleLowerCase() === storeId
				);
				if (!!store) {
					updatedStoresToAdd = storesToAdd.filter(
						s => s.store.id.toLocaleLowerCase() !== storeId
					);
				} else {
					noUpdate = true;
					await StoreHelper.removeStore({
						id: storeId,
						drawerClass,
						listItemType,
						stateUser,
						noUpdate,
					});
				}
			}

			await State.updateUserToAdd({
				storesToAdd: updatedStoresToAdd,
			});

			if (noUpdate) return;

			await StoreHelper.updateStoresChips({
				drawerElement,
				drawerClass,
				stateUser,
				stores: updatedStoresToAdd,
			});
		}
	},
	toggleStoreCheckboxAndAddEventListener: function ({
		checkboxes,
		drawerElement,
		drawerClass,
		stores,
		stateUser,
		listItemType,
	}) {
		Array.from(checkboxes).forEach(cb => {
			const { MDCCheckbox } = mdc.checkbox;

			const {
				root: { firstElementChild: checkBoxElm },
			} = MDCCheckbox.attachTo(cb);
			const { id } = checkBoxElm;
			const storeId = Utils.extractStoreIdFromCheckBoxId(id);

			Utils.toggleCheckbox({
				mdcCheckboxDomElm: cb,
				toggle: stores
					?.filter(s => Boolean(s))
					?.some(s => s.store.id === storeId),
			});

			cb.addEventListener(
				'change',
				async e =>
					await UserFlow.storeCheckBoxListenerFunction(
						e,
						drawerElement,
						drawerClass,
						stateUser,
						listItemType
					)
			);
		});
	},
	filterCountriesWithStores: async function ({ countries, brand }) {
		if (!countries?.length) return [];

		const body = {
			level: UserAccessInfoLevels.COUNTRIES,
			brand,
		};
		const { data: loggedInUserCountries = [] } = await Cache.cachedFunction(
			Cache.constructUserAccessInfoCacheKey(body),
			1,
			async () => await Api.getUserAccessInfo(body)
		);

		return countries.filter(
			c =>
				!!c.name && loggedInUserCountries.some(countryId => countryId === c.id)
		);
	},
	filteredStores: async function ({ brand, country, stores }) {
		if (!stores?.length) return [];

		const body = {
			level: UserAccessInfoLevels.STORES,
			countryCode: country,
			brand,
		};
		const { data: loggedInUserStores = [] } = await Cache.cachedFunction(
			Cache.constructUserAccessInfoCacheKey(body),
			1,
			async () => await Api.getUserAccessInfo(body)
		);

		return stores.filter(
			s =>
				!!s.name &&
				loggedInUserStores.some(
					storeId => storeId.toLocaleLowerCase() === s.id.toLocaleLowerCase()
				)
		);
	},
	getCurrentUserHighestUserRole: function ({ loggedInUserRoles }) {
		if (!loggedInUserRoles) return {};

		const allRoles = Utils.getAllRoles();

		return Object.values(loggedInUserRoles).reduce((highestRole, current) => {
			const currentUserRole = Utils.getUserRoleByName(current[0].role);
			return !highestRole || currentUserRole?.level > highestRole.level
				? currentUserRole
				: highestRole;
		}, allRoles.values().next().value);
	},
	handleUpdateUser: async function ({
		operations,
		loader = {
			show: () => {},
			hide: () => {},
		},
		isMultipleAdd = false,
		loaderText = 'Updating stores and roles',
	}) {
		if (!operations || !operations.length) {
			return false;
		}
		let error = false;
		let errorMessageArray = [];

		loader.show(loaderText);
		try {
			for (const operation of operations.sort((a, _) =>
				a.operation === UserBatchOperation.REMOVE ? -1 : 1
			)) {
				const { statusCode = 500, statusMessage } = await Api.updateUser({
					operation,
				});
				const ok = statusCode === 200;
				await UserFlow.pushUserOperationMessageToToastQueue({
					operation,
					success: ok,
				});
				error = !ok;
				if (!ok) {
					errorMessageArray.push(statusMessage);
				}
			}
		} catch (error) {
			error = true;
		}
		await State.updateUserToAdd({
			storesToAdd: [],
			storesToRemove: [],
		});

		if (!error) {
			setTimeout(() => {
				if (!isMultipleAdd) {
					window.location.reload(true);
				}
			}, 6000);
			return true;
		} else {
			loader.hide();
			let toastMessage = 'Something went wrong! Please try again.';
			if (errorMessageArray.length) {
				toastMessage = errorMessageArray.join(', ');
			}
			const toast = new Toast({
				data: {
					toastMessage,
				},
			});
			toast.open();

			const drawerElement = document.querySelector(
				DrawerClasses.ADD_USER__ADD_VALUES
			);

			await DomHelper.updateStoresList({
				drawerClass: DrawerClasses.ADD_USER__ADD_VALUES,
				stateUser: StateKeys.USER_TO_ADD,
				stores: [],
				listItemType:
					StoreHelper.StoreListItemKeys.STORE_LIST_ITEM_WITH_CHECKBOX,
				disableCurrentStores: true,
			});

			await StoreHelper.updateStoresChips({
				drawerElement,
				drawerClass: DrawerClasses.ADD_USER__ADD_VALUES,
				stateUser: StateKeys.USER_TO_ADD,
				stores: [],
				isPending: true,
			});

			Utils.setSelectsValue({
				selects: drawerElement.querySelectorAll('.mdc-select'),
				dataObj: {
					role: INITIAL_VALUE,
				},
			});

			await UserFlow.closeAllOpenDrawers();

			return false;
		}
	},
	pushUserOperationMessageToToastQueue: async function ({
		operation,
		success,
	}) {
		const {
			operation: o,
			role: r,
			brand: b,
			countryCode,
			storeId,
			userToWorkWithEmail,
		} = operation;
		const { data: countrySpecifics } = Cache.getCache(CacheKeys.COUNTRY_DATA);
		const operationText = o === UserBatchOperation.ADD ? 'added' : 'removed';
		const resultText = success ? 'successfully!' : 'failed!';
		const brand = Utils.brandsMap[b]?.name;
		const country = countrySpecifics[countryCode]?.name;
		const role = Array.from(Utils.getAllRoles()).find(x => x.id === r)?.name;

		const message = `${role} for ${storeId.toUpperCase()} in ${brand}, ${country} ${operationText} to ${userToWorkWithEmail} ${resultText}`;
		await Toast.externalAddtoQueue({ message });
	},
	handleAddUser: async function ({ loader }) {
		const {
			storesToAdd = [],
			storesToRemove = [],
			eMail,
			userPrincipalName,
		} = await State.getSingleOrMultipleToAddUser();

		const email = UserFlow.getEmailFromUser({
			eMail,
			userPrincipalName,
		});

		const operations = await UserFlow.createOperations({
			stateArray: [
				{ values: storesToAdd, operationType: UserBatchOperation.ADD },
				{
					values: storesToRemove,
					operationType: UserBatchOperation.REMOVE,
				},
			],
			userToWorkWithEmail: email,
		});

		await State.setUsersState({
			userToAdd: {},
		});

		await UserFlow.handleUpdateUser({
			operations,
			loader,
		});
	},
	createOperations: async function ({ stateArray, userToWorkWithEmail }) {
		const operations = [];

		stateArray?.forEach(({ values, operationType }) => {
			values?.forEach(val =>
				operations.push(
					Api.createBatchOperationObject({
						store: val,
						operationType,
						userToWorkWithEmail,
					})
				)
			);
		});

		return operations;
	},
	closeAllOpenDrawers: async function () {
		const drawers = await State.getState(StateKeys.DRAWERS);
		Object.keys(drawers).forEach(async d => {
			const [drawerKey, drawerId] = Object.entries(DrawerKeys).find(
				([_, id]) => id === d
			);
			const [_, drawerClass] = Object.entries(DrawerClasses).find(
				([key, _]) => key === drawerKey
			);
			const drawer = document.querySelector(drawerClass);
			const isOpen = Utils.isDrawerOpen({ drawer });

			if (isOpen) {
				const { hide } = await State.getDrawerById(drawerId);
				hide();
			}
		});
	},
	toggleMultipleAddSaveBtn: async function () {
		const { storesToAdd } = await State.getSingleOrMultipleToAddUser();
		const $addUserAddValuesDrawer = document.querySelector(
			DrawerClasses.ADD_USER__ADD_VALUES
		);
		const $multipleAddSaveBtn = $addUserAddValuesDrawer.querySelector(
			'.multiple-add-save-button'
		);
		$multipleAddSaveBtn[
			!!storesToAdd?.length ? 'removeAttribute' : 'setAttribute'
		]('disabled', 'disabled');
	},
	handleMultipleAddSaveBtnClick: async function ({
		loader = {
			show: () => {},
			hide: () => {},
		},
	}) {
		const { storesToRemove = [], storesToAdd = [] } =
			await State.getSingleOrMultipleToAddUser();
		const { usersToAdd = [] } = await State.getState(StateKeys.USERS);

		loader.show('Adding users...');
		const userUpdates = await Promise.all(
			usersToAdd?.map(async u => {
				const { eMail, userPrincipalName } = u;
				const email = UserFlow.getEmailFromUser({
					eMail,
					userPrincipalName,
				});

				const operations = await UserFlow.createOperations({
					stateArray: [
						{ values: storesToAdd, operationType: UserBatchOperation.ADD },
						{
							values: storesToRemove,
							operationType: UserBatchOperation.REMOVE,
						},
					],
					userToWorkWithEmail: email,
				});
				return await UserFlow.handleUpdateUser({
					operations,
					loader,
					multipleAdd: true,
				});
			})
		);

		if (userUpdates.some(u => !u)) {
			return;
		}
		await State.updateUserToAdd({ storesToRemove: [], storesToAdd: [] });
		await State.setUsersState({
			usersToAdd: [],
			userToAdd: {},
			multipleAdd: false,
		});

		await StoreHelper.updateStoresChips({
			drawerElement: document.querySelector(DrawerClasses.ADD_USER__ADD_VALUES),
			drawerClass: DrawerClasses.ADD_USER__ADD_VALUES,
			stateUser: StateKeys.USER_TO_ADD,
			stores: [],
			isPending: true,
		});
	},
	handleInviteUser: async function () {
		const user = await State.getSingleOrMultipleToAddUser();
		const { eMail, givenName, origin, userPrincipalName, userLocation } = user;

		if (origin?.toLocaleLowerCase() === UserOrigin.HM) {
			const { brand, countryCode, storeId } = await State.getState(
				StateKeys.CURRENT_USER
			);

			const email = UserFlow.getEmailFromUser({ eMail, userPrincipalName });

			const userToInvite = {
				...user,
				invitationType: 'EntraID',
				eMail: email,
				firstName: givenName,
				storeID: storeId,
				usageLocationCountryCode: userLocation,
				brand,
				countryCode,
			};

			await Api.inviteUser(userToInvite);
		}
		return user;
	},
	checkIfUserIsInvited: async function () {
		const { eMail, userPrincipalName } =
			await State.getSingleOrMultipleToAddUser();

		const email = UserFlow.getEmailFromUser({ eMail, userPrincipalName });

		while (true) {
			const user = await Api.getUserByEmail(email);
			if (user?.id && !user?.error) {
				return true;
			}
			await new Promise(resolve => setTimeout(resolve, 3000));
		}
	},
	updateAddValuesDrawerStoresListAndCheckboxes: async function ({
		drawerElement,
		role,
		brand,
		country,
	}) {
		const {
			storesAndRoles = [],
			storesToAdd = [],
			roles,
			storesAndRolesToAdd = [],
			role: roleFromUserToManage = {},
		} = await State.getSingleOrMultipleToAddUser();

		const r = !!role?.name ? role : roleFromUserToManage;

		const extendedStoresList = await StoreHelper.getExtendedStoresList({
			userToManageStoresAndRoles: storesAndRoles,
			storesToAdd,
			role: r,
			brand,
			country,
		});

		await DomHelper.updateStoresList({
			drawerClass: DrawerClasses.ADD_USER__ADD_VALUES,
			stateUser: StateKeys.USER_TO_ADD,
			stores: extendedStoresList,
			checkRole: true,
			role: r?.id,
			listItemType: StoreHelper.StoreListItemKeys.STORE_LIST_ITEM_WITH_CHECKBOX,
		});

		const form = drawerElement.querySelector('form');
		const checkboxes = form.querySelectorAll('.mdc-checkbox');

		UserFlow.toggleStoreCheckboxAndAddEventListener({
			checkboxes,
			drawerClass: DrawerClasses.ADD_USER__ADD_VALUES,
			drawerElement,
			stateUser: StateKeys.USER_TO_ADD,
			stores: extendedStoresList?.filter(s => s.existOnUser),
			listItemType: StoreHelper.StoreListItemKeys.STORE_LIST_ITEM_WITH_CHECKBOX,
		});

		await UserFlow.updateUserProfileDrawerRolesAndStores({
			drawerElement: document.querySelector(
				DrawerClasses.ADD_USER__USER_PROFILE
			),
			roles: { ...storesAndRolesToAdd, ...roles },
		});
	},
	handleNextModalTriggers: async function ({
		_nextModalTriggers,
		_nextModal,
		_forceOnCallback,
		_callback,
		form,
		_submitBtnLoader,
		loader,
		hide,
	}) {
		if (_nextModalTriggers && _nextModal) {
			const d = await State.getDrawerById(_nextModal);
			const { show: showNextModal } = d;

			if (d) {
				Array.from(_nextModalTriggers).map(async nextModalTrigger => {
					if (!nextModalTrigger.classList.contains('next-modal-trigger')) {
						return false;
					}
					if (!nextModalTrigger.dataset.listener) {
						nextModalTrigger.addEventListener('click', async () => {
							nextModalTrigger.setAttribute('disabled', 'disabled');

							if (_forceOnCallback) {
								await _callback({
									formData: form ?? {},
									submitBtnLoader: _submitBtnLoader,
									loader,
									hide,
								});
							}
							if (showNextModal) showNextModal();
							nextModalTrigger.removeAttribute('disabled');
						});
						nextModalTrigger.dataset.listener = true;
					}
				});
			}
		}
	},
	handleUserResetPassword: async function ({
		userId,
		mail,
		userName,
		loader = { show: () => {}, hide: () => {} },
	}) {
		if (!userId) return false;
		loader.show();
		const {
			data: { newPassword = '' },
			statusCode,
		} = await Api.resetPassword(userId);

		const successfull = statusCode === 200;

		const confirmMessage = successfull
			? `Password reset for user ${userName} (${mail}) successfull. <br /><br />The new password is: <b>${newPassword}</b>. <br /><br />Please save this password, you wont be able to see it again.`
			: 'Something went wrong resetting password. Please try again.';

		const title = successfull ? 'Success' : 'Failure';

		const confirmDialog = new Dialog({
			title,
			content: `<p class="mdc-theme--primary mdc-typography--body1">${confirmMessage}</p>`,
			confirmButtonText: 'OK',
			closeButtonText: '',
			confirmAction: () => {},
		});

		loader.hide();
		confirmDialog.open();
	},
	fetchAndRenderUserList: async function ({
		loaderShow = false,
		loader,
		searchValue,
		defaultValue,
		selectedStore,
		searchTerm = UserSearchSearchTerms.STORE_USERS,
	}) {
		if (loaderShow) {
			loader.show();
		}

		const isPagination = searchTerm === UserSearchSearchTerms.PAGINATION;

		const _searchVal = searchValue || defaultValue;

		const {
			data: users = [],
			numberOfRecords = 0,
			nextPage = '',
		} = await Api.getStoreUsers(
			isPagination
				? {
						searchTerm,
						nextLink: searchValue,
				  }
				: {
						searchValue: searchValue || defaultValue,
						searchTerm,
						selectedStoreGroupName: selectedStore,
				  }
		);

		DomHelper.updateUsersList({
			users,
			loader,
			searchValue: _searchVal === defaultValue ? '' : _searchVal,
			searchTerm,
			numberOfRecords,
			nextPage,
		});
		loader.hide();
	},
	renderApplicationsSwitches: async function ({ list }) {
		const { apps: applications } = await Cache.cachedFunction(
			CacheKeys.APPLICATIONS,
			3,
			async () => await Api.getGlobalApplications()
		);
		const applicationsFromLoggedInUsers = ['admintool', 'backspace'];
		let LISTITEMS = '';

		applications.forEach(app => {
			const appIdString = `applications-switch-${app?.aadGroupName}`;
			const btnSelectedClass = app?.default
				? 'mdc-switch--selected'
				: 'mdc-switch--unselected';
			LISTITEMS += `<li class="flex justify-between w-full">
								<label class="mdc-typography--body2 mb-0 font-normal flex items-center" for="${appIdString}">${app.name}</label>
								<button
									class="mdc-switch !w-[52px] !h-8 ${btnSelectedClass}"
									type="button"
									role="switch"
									aria-checked="${app?.default}"
									id="${appIdString}"
								>
									<div class="mdc-switch__track !h-full !rounded-2xl"></div>
									<div class="mdc-switch__handle-track">
										<div class="mdc-switch__handle">
											<div class="mdc-switch__shadow">
												<div class="mdc-elevation-overlay"></div>
											</div>
											<div class="mdc-switch__ripple"></div>
											<div class="mdc-switch__icons">
												<svg
													class="mdc-switch__icon mdc-switch__icon--on"
													viewBox="0 0 24 24"
												>
													<path
														d="M19.69,5.23L8.96,15.96l-4.23-4.23L2.96,13.5l6,6L21.46,7L19.69,5.23z"
													/>
												</svg>
												<svg
													class="mdc-switch__icon mdc-switch__icon--off"
													viewBox="0 0 24 24"
												>
													<path d="M20 13H4v-2h16v2z" />
												</svg>
											</div>
										</div>
									</div>
								</button>
							</li>`;
		});

		list.innerHTML = LISTITEMS;
		initMDCSwitches();
	},
};
